"""
Test our Zenodo interaction flows
"""

from __future__ import annotations

import json
import os

import pytest
import requests

from openscm_zenodo.zenodo import ZenodoDomain, ZenodoInteractor


@pytest.mark.zenodo_token
def test_default_end_to_end_flow(test_data_dir):
    """
    Test we can start with an ID and end up publishing a new version
    """
    any_deposition_id = "101709"
    metadata_file = test_data_dir / "test-deposit-metadata.json"
    sub_dir_file = test_data_dir / "sub-dir" / "file-in-sub-dir.txt"
    files_to_upload = [metadata_file, sub_dir_file]

    zenoodo_interactor = ZenodoInteractor(
        token=os.environ["ZENODO_TOKEN"],
        zenodo_domain=ZenodoDomain.sandbox.value,
    )

    latest_deposition_id = zenoodo_interactor.get_latest_deposition_id(
        any_deposition_id=any_deposition_id,
    )

    # Sometimes you have to delete drafts manually by hand before this works.
    # I'm not sure why this happens,
    # the sandbox seems to somehow end up in a weird state.
    new_deposition_id = zenoodo_interactor.create_new_version_from_latest(
        latest_deposition_id=latest_deposition_id
    ).json()["id"]

    # Optional, you might want the previous version's files in some cases
    remove_all_files_responses = zenoodo_interactor.remove_all_files(
        deposition_id=new_deposition_id
    )
    assert all(
        isinstance(response, requests.models.Response)
        for response in remove_all_files_responses
    )

    with open(metadata_file) as fh:
        metadata = json.load(fh)

    update_metadata_response = zenoodo_interactor.update_metadata(
        deposition_id=new_deposition_id,
        metadata=metadata,
    )
    assert isinstance(update_metadata_response, requests.models.Response)

    bucket_url = zenoodo_interactor.get_bucket_url(deposition_id=new_deposition_id)

    for file in files_to_upload:
        resp = zenoodo_interactor.upload_file_to_bucket_url(
            file,
            bucket_url=bucket_url,
        )
        assert isinstance(resp, requests.models.Response)

    publish_response = zenoodo_interactor.publish(deposition_id=new_deposition_id)
    assert isinstance(publish_response, requests.models.Response)

    publish_response_json = publish_response.json()

    assert publish_response_json["id"] == new_deposition_id
    comparable_metadata_from_publish_response = {
        "metadata": {
            k: v
            for k, v in publish_response_json["metadata"].items()
            if k in metadata["metadata"]
        }
    }
    assert comparable_metadata_from_publish_response == metadata

    assert len(publish_response_json["files"]) == len(files_to_upload)
    # Zenodo doesn't support directories, so uploaded files should be flat.
    # Zipping files is the work around apparently.
    # See https://support.zenodo.org/help/en-gb/1-upload-deposit/74-can-i-upload-folders-directories
    publish_response_uploaded_files = [
        file_record["filename"] for file_record in publish_response_json["files"]
    ]
    assert set(publish_response_uploaded_files) == set(
        [f.name for f in files_to_upload]
    )



"""
Logging
"""

from __future__ import annotations

import io
import sys
from pathlib import Path
from typing import Any, Optional, Union

from loguru import logger
from typing_extensions import TypeAlias

LoggingConfigType: TypeAlias = Union[dict[str, list[dict[str, Any]]], None]


def get_default_config(
    level: str = "INFO",
) -> dict[str, list[dict[str, Union[Union[io.TextIOWrapper, Any], str, bool]]]]:
    """
    Get default logging configuration

    Parameters
    ----------
    level
        Level to apply to the logging

    Returns
    -------
    :
        Default logging configuration
    """
    return dict(
        handlers=[
            dict(
                sink=sys.stderr,
                level=level,
                colorize=True,
                format=" - ".join(
                    [
                        "{process}",
                        "{thread}",
                        "<green>{time:!UTC}</>",
                        "<level>{level}</>",
                        "<cyan>{name}:{file}:{line}</>",
                        "<level>{message}</>",
                    ]
                ),
            )
        ],
    )


def setup_logging(
    enable: bool,
    logging_config: Optional[Union[Path, LoggingConfigType]] = None,
    logging_level: Optional[str] = None,
) -> None:
    """
    Set up logging

    Parameters
    ----------
    enable
        Whether to enable the logger.

        If `False`, we explicitly disable logging,
        ignoring the value of all other arguments.

    logging_config
        If a `dict`, passed to
        [`loguru.logger.configure`][loguru._logger.Logger.configure].
        If not passed, [`get_default_config`][openscm_zenodo.logging.get_default_config]
        is used.
        Otherwise, we try and load this from disk using
        [`loguru_config.LoguruConfig`](https://github.com/erezinman/loguru-config).

        This takes precedence over `log_level`.

    logging_level
        Log level to apply to the default config.
    """
    if not enable:
        # Should already be disabled, but just in case
        logger.disable("openscm_zenodo")
        return

    if logging_config is None:
        if logging_level is not None:
            config = get_default_config(level=logging_level)
        else:
            config = get_default_config()

        # Not sure what is going on with type hints, one for another day
        logger.configure(handlers=config["handlers"])

    elif isinstance(logging_config, dict):
        # mypy not happy about kwargs being passed here,
        # fair enough I guess
        logger.configure(**logging_config)  # type: ignore

    else:
        # Type ignore while we wait for new release of loguru-config
        try:
            from loguru_config import LoguruConfig  # type: ignore
        except ImportError:
            msg = (
                "[loguru-config](https://github.com/erezinman/loguru-config) "
                "is required to load config from disk. "
                "Run `pip install loguru-config`."
                "If that doesn't work, see installation instructions here: "
                "https://github.com/erezinman/loguru-config#installation"
            )
            print(msg)

            raise

        loguru_configurer = LoguruConfig.load(logging_config, configure=False)
        loguru_configurer.load()

    logger.enable("openscm_zenodo")

    if logging_config is not None and logging_level is not None:
        logger.warning("`logging_level` is ignored if `logging_config` is supplied")


def mask_token(input: str, token: Union[str, None]) -> str:
    """
    Mask any token values in `input`

    Parameters
    ----------
    input
        Value in which the token values should be masked

    token
        Token value to mask

        If not supplied, this function becomes a no-op.

    Returns
    -------
    :
        `input` with `token` masked
    """
    if token is None:
        return input

    return input.replace(token, "***")
